/********************************************************************
 * 2D Map for Radarcape
 ********************************************************************
 * Guenter Koellner Embedded Development GmbH
 *
 * Am Rain 24
 * 85256 Vierkirchen
 * Geschaeftsfuehrer Guenter Koellner
 *
 * http://www.modesbeast.com
 ********************************************************************/

var Map=null;
var Planes={};
var Filters={altitude: [undefined, undefined],
             enable_alt:  undefined,
             speed: [undefined, undefined],
             enable_spd:  undefined,
             distance: [undefined, undefined],
             enable_dis:  undefined,
             typ_enable:  undefined,
             typ_filter:  undefined,
             fli_enable:  undefined,
             fli_filter:  undefined,
             tracklength: undefined,
             fleetwatch:  undefined,
             reg_enable:  undefined,
             refreshtime: undefined
            };
var NumPlanes = 0;
var Selected=null;
var RangeCircle={};
var CircleStep=50;
var NauticalMiles_to_Meters=1852;
var KiloMeters_to_Meters=1000;

//var map_container;
var map_box;
var map_info_box;
var flight_info_box;
var hide_info;
var hide_info_btn;

function showInfoBox()
{
    hide_info.style.visibility='visible';
    map_info_box.style.width='210px';
    hide_info_btn.setAttribute("class", "glyphicon glyphicon-minus");
}

function hideInfoBox()
{
    hide_info.style.visibility='hidden';
    map_info_box.style.width='1px';
    hide_info_btn.setAttribute("class", "glyphicon glyphicon-plus");
}

function toggleInfoBox()
{
  if(hide_info.style.visibility == 'hidden')
    showInfoBox();
  else
    hideInfoBox();
}

function get_image_object(ac_category, ac_type)
{
   var retVal = acImagePathsR["$0"];

   return retVal;
}


function getIconForPlane(plane)
{

      var rFill   = 255; gFill   = 255; bFill = 0;
      var rBorder = 64,  gBorder = 64,  bBorder = 64;

      if (plane.timeout == 1) /* after timeout 1 */
      {
          rFill = 128; gFill = 128; bFill = 128;  /* dark gray */
          rBorder = rFill; gBorder=gFill; bBorder=bFill;
      }

      var maxalt = 40000; /* Max altitude in the average case */
      var invalt = maxalt-plane.alt;
      var selected = (Selected == plane.id);
      var imageObj = get_image_object("dummy1", "dummy2");

      var object_box = Raphael.pathBBox(imageObj.path);
      var offset_x = (object_box.x + object_box.x2) / 2;
      var offset_y = (object_box.y + object_box.y2) / 2;

      if (invalt < 0) invalt = 0;

      var strokew = 1;
      if (selected) { strokew = 2; }
      if (plane.timeout == 2) { strokew = 0.333; }   /* light on  timeout 2 */

      var ac_symbol = {
         strokeColor: 'rgb('+rBorder+','+gBorder+','+bBorder+')',
         strokeWeight: strokew,
         path: imageObj.path,
         scale: imageObj.scale,
         fillColor: 'rgb('+rFill+','+gFill+','+bFill+')',
         fillOpacity: 0.9,
         anchor: new google.maps.Point(offset_x, offset_y),
         rotation: (plane.trk + imageObj.rotcorr) % 360
      };

      var myZoom = Map.getZoom();
      if((myZoom <= 14) && ((6 < myZoom))) {
         ac_symbol.scale = ac_symbol.scale + 0.2 * ((myZoom-10)/4);
      }
      else if(myZoom < 14) {
         ac_symbol.scale = ac_symbol.scale - 0.2
      }
      else {
         ac_symbol.scale = ac_symbol.scale + 0.2
      }

      return ac_symbol;
}


function selectPlane()
{
      if (!Planes[this.planehex])
        return;

      var Old_Selected = Selected;
      Selected = this.planehex;

      if (Planes[Old_Selected])
      {
         /* Remove the highlight in the previously selected plane. */
         Planes[Old_Selected].marker1.setIcon(getIconForPlane(Planes[Old_Selected]));
         labelSetStyle(Planes[Old_Selected], Planes[Old_Selected].marker1);
      }

      Planes[Selected].marker1.setIcon(getIconForPlane(Planes[Selected]));
      labelSetStyle(Planes[Selected],Planes[Selected].marker1);

      refreshSelectedInfo();
      showInfoBox();
}


function deselectPlane()
{
     if (Selected) {
         var Old_Selected = Selected;
         Selected = null;
         Planes[Old_Selected].marker1.setIcon(getIconForPlane(Planes[Old_Selected]));
         labelSetStyle(Planes[Old_Selected],Planes[Old_Selected].marker1);
         flight_info_box.innerHTML = "";
      }
}


function sortPlaneSymbolList(a, b)
{
      var al = a.id;
      var bl = b.id;

      if (al < bl) return 1;
      if (al > bl) return -1;
      return 0;
}


/* Check if an aircraft disappeared and therefore was deselected */
function checkAircraftDeselected()
{
  if (Selected == null)
  {
  }
}

function getTimeString(isLocal) {
  var hours;
  var minutes;
  var date = new Date();
  var zone_offset = date.getTimezoneOffset();

  if(isLocal)
  {
    hours = date.getHours();
    minutes = date.getMinutes();
  }
  else
  {
    hours = date.getUTCHours();
    minutes = date.getUTCMinutes();
  }

  if(hours   < 10) {hours = '0'+hours;}
  if(minutes < 10) {minutes = '0'+minutes;}

  var zeit_string = hours + ':' + minutes;

  return zeit_string;
}

/* Info box in div map_info_box - 0=Aircraft Info 1=Config */
function refreshGeneralInfo()
{
  var html = '';

  html += "<table class=\"info_box_table\">";
  html += "  <thead>";

  html += "    <tr class=\"info_header\">";
  html += "      <td colspan=\"2\"><span class=\"glyphicon glyphicon-time\" aria-hidden=\"true\"></span> General Information</td>";
  html += "    </tr>";

  html += "  </thead>";
  html += "  <tbody>";

  html += "    <tr class=\"info_label\">";
  html += "      <td>UTC Time</td>";
  html += "      <td>Local Time</td>";
  html += "    </tr>";

  html += "    <tr class=\"info_value\">";
  html += "      <td>" + getTimeString(false) + "</td>";
  html += "      <td>" + getTimeString(true) + "</td>";
  html += "    </tr>";

  html += "    <tr class=\"info_label\">";
  html += "      <td colspan=\"2\">Number of Aircraft on Map</td>";
  html += "    </tr>";

  html += "    <tr class=\"info_value\">";
  html += "      <td colspan=\"2\">" + NumPlanes +"</td>";
  html += "    </tr>";

  html += "  </tbody>";
  html += "</table>";

  document.getElementById('general_info_box').innerHTML = html;
}


function refreshSelectedInfo()
{
  var p = undefined;

  if(Selected)
    p = Planes[Selected];

  if (p && Selected)
  {
    var html = '';

    html += '<table class="info_box_table">';

    /* AC Details Header */
    html += '    <tr class="info_header">';
    html += '      <td colspan="2"><span class="glyphicon glyphicon-plane" aria-hidden="true"></span> Aircraft Details</td>';
    html += '    </tr>';

    /* AC Details Row 1 */
    html += '  <tr class="info_label">';
    html += '    <td width="50%">Registration</td>';
    html += '    <td width="50%">Mode-S</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    if(p.reg != "") {
      html += '    <td><a href="http://www.planespotters.net/Aviation_Photos/search.php?tag='
           + encodeURIComponent(p.reg) + '" target="_blank">' + p.reg + '</a>&nbsp;</td>';
    } else {
      html += '    <td>' + p.reg + '&nbsp;</td>';
    }
    html += '    <td>'+ p.id +'&nbsp;</td>';
    html += '  </tr>';

    /* AC Details Row 2 */
    html += '  <tr class="info_label">';
    html += '    <td>Type</td>';
    html += '    <td>Category</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.typ + '&nbsp;</td>';
    html += '    <td>' + p.cat + '&nbsp;</td>';
    html += '  </tr>';

    /* Flight Details Header */
    html += '    <tr class="info_header">';
    if (p.fli.length) {
      html += '    <td colspan="2"><span class="glyphicon glyphicon-plane" aria-hidden="true"></span> Flight '+ p.fli +'</td>';
    } else {
      html += '    <td colspan="2"><span class="glyphicon glyphicon-plane" aria-hidden="true"></span> Flight Details</td>';
    }
    html += '    </tr>';

    /* Flight Details Row 1 */
    html += '  <tr class="info_label">';
    html += '    <td>Origin</td>';
    html += '    <td>Destination</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.org + '&nbsp;</td>';
    html += '    <td>' + p.dst + '&nbsp;</td>';
    html += '  </tr>';

    /* Flight Details Row 2 */
    html += '  <tr class="info_label">';
    html += '    <td>Track</td>';
    html += '    <td>Speed</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.trk + '&deg;</td>';
    html += '    <td>' + p.spd + ' kt</td>';
    html += '  </tr>';

    /* Flight Details Row 3 */
    html += '  <tr class="info_label">';
    html += '    <td>Altitude</td>';
    html += '    <td>Vertical Rate</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.alt + ' ft</td>';
    if(p.vrt) {
      html += '    <td>' + p.vrt + ' ft/min</td>';
    } else {
      html += '    <td></td>';
    }
    html += '  </tr>';

    /* AC Tracking Header */
    html += '    <tr class="info_header">';
    html += '      <td colspan="2"><span class="glyphicon glyphicon-map-marker" aria-hidden="true"></span> Aircraft Tracking</td>';
    html += '    </tr>';

    /* AC Tracking Row 1 */
    html += '  <tr class="info_label">';
    html += '    <td>Latitude</td>';
    html += '    <td>Longitude</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.lat + '&deg;</td>';
    html += '    <td>' + p.lon + '&deg;</td>';
    html += '  </tr>';

    /* AC Tracking Row 2 */
    html += '  <tr class="info_label">';
    html += '    <td>Distance</td>';
    html += '    <td>Source</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + "@PH1" + ' ' + DistanceUnit + '</td>';
    html += '    <td>@PH2</td>';
    html += '  </tr>';

    /* AC Tracking Row 3 */
    html += '  <tr class="info_label">';
    html += '    <td>Last Signal</td>';
    html += '    <td>&nbsp;</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.lla + ' sec</td>';
    html += '    <td>&nbsp;</td>';
    html += '  </tr>';

    html += '</table>';

    flight_info_box.innerHTML = html;
  }
  else
  {
    /* Remove Selected Info */
    flight_info_box.innerHTML = "<p>Click on a client for more info.</p>";
  }
}

// This is the text that appears with all displayed aircraft
function labelDataTable(plane)
{
    var labelDataTab = "<table class=\"aircraft_label\">" + 
                       "<tr><td class=\"flight\">" + plane.id + "</td><td class=\"altitude\">" + plane.ac + "</td></tr>" +
		       "<tr><td class=\"verticalRate\">" + plane.iBy + "</td><td class=\"speed\">" + plane.iMsg + "</td></tr>" +
		       "</table>";
  return labelDataTab;
}


function labelSetAnchor(plane, marker, offset)
{
  if (plane.trk > 315) { marker.set("labelAnchor", new google.maps.Point( 66, -1 + offset)); return; }
  if (plane.trk < 45)  { marker.set("labelAnchor", new google.maps.Point( 66, -1 + offset)); return; }
  if (plane.trk < 135) { marker.set("labelAnchor", new google.maps.Point( 37, -1 + 28 + offset)); return; }
  if (plane.trk < 225) { marker.set("labelAnchor", new google.maps.Point(-17, -1 + offset)); return; }
  marker.set("labelAnchor", new google.maps.Point(37, -1 - 28 + offset));
}


function labelSetStyle(plane, marker)
{
      if (plane.id  == Selected) {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'greenyellow'}); return; }
      if (plane.timeout == 1)    {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'silver'}); return; }
      if (plane.timeout == 2)    {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'tan'}); return; }
      if (plane.src == "M")      {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orange'}); return; }
      marker.set("labelStyle", {opacity: 0.6, 'background-color': 'gold'});
}


function fetchData() {
  
    if ( Map.getBounds()==null) {
      return;
    }

    var vps = Map.getBounds().getSouthWest().lat().toFixed(2);
    var vpw = Map.getBounds().getSouthWest().lng().toFixed(2);

    var vpn = Map.getBounds().getNorthEast().lat().toFixed(2);
    var vpe = Map.getBounds().getNorthEast().lng().toFixed(2);
    var viewpoint = '&vpn='+vpn+'&vpe='+vpe+'&vps='+vps+'&vpw='+vpw

    var latcntr = Map.getCenter().lat().toFixed(2);
    var loncntr = Map.getCenter().lng().toFixed(2);

    var alt_min = "";
    var alt_max = "";
    if ( Filters.enable_alt ) {
        if ( typeof Filters.altitude[0]!=="undefined" && Filters.altitude[0]!==null ) {
            alt_min = "&altitude.from=" + Filters.altitude[0];
        }

        if ( typeof Filters.altitude[1]!=="undefined" && Filters.altitude[1]!==null ) {
            alt_max = "&altitude.to=" + Filters.altitude[1];
        }
    }

    $.getJSON('/mlat/clients.json',
      function(data_unfiltered) {
        var stillhere = {}
        var data = data_unfiltered["TLS"];
        console.log("fetch: " + data.length);
        for (var j=0; j < data.length; j++) {
            var plane = data[j];
            var marker1 = null;

            stillhere[plane.id] = true;
            plane.fli = $.trim(plane.fli);

            /* update */
            if (Planes[plane.id]) {
                marker1 = Planes[plane.id].marker1;

                var newpos = new google.maps.LatLng(plane.lat, plane.lon);

                /* update flight number */
                marker1.setIcon(getIconForPlane(plane));
                marker1.setPosition(newpos);
                marker1.set("labelContent", labelDataTable(plane));
                labelSetAnchor(plane, marker1, 12);
                labelSetStyle(plane, marker1);
                marker1.label.draw();

                if (plane.id == Selected)
                    refreshSelectedInfo();

            } else {
                /* new flight */
                marker1 = new MarkerWithLabel({
                    position: new google.maps.LatLng(plane.lat, plane.lon),
                        draggable: false,
                        raiseOnDrag: false,
                        zIndex: 70000,
                        map: Map,
                        labelContent: labelDataTable(plane),
                        labelAnchor: new google.maps.Point(0,0),
                        labelClass: "labels",
                        labelStyle: {},
                        labelZIndex: 60000,
                        icon: getIconForPlane(plane)
                });
                labelSetAnchor(plane,marker1,12);
                    labelSetStyle(plane,marker1);

                    /* Trap clicks for this marker. */
                    marker1.addListener('click', selectPlane);
            }

            marker1.planehex = plane.id;   /* for click */
            plane.marker1 = marker1;
            Planes[plane.id] = plane;

            if (plane.fli.length == 0) {
                marker1.setTitle(plane.id)
            } else {
                /* Popup window when mouse over aircraft icon */
                var title = plane.reg+' ('+plane.id+')';
                if (plane.org.length && plane.dst.length)
                    title += '\n'+plane.org+'>'+plane.dst;
                    marker1.setTitle(title)
            }
        }
        NumPlanes = data.length;

        /* Remove idle planes. */
        for (var p in Planes) {
            if (! stillhere[Planes[p].id]) {
                if(Planes[p].id == Selected) {
                    deselectPlane();
                }
                Planes[p].marker1.setMap(null);
                delete Planes[p];
            }
        }
    });
}

function FilterControl(controlDiv, map)
{
    var controlUI = document.createElement('div');
    controlUI.className = 'filter-button';
    controlUI.title = 'Click to modify the filters';
    controlDiv.appendChild(controlUI);

    var controlText = document.createElement('div');
    controlText.className = 'filter-icon';
    controlUI.appendChild(controlText);

    controlUI.addEventListener('click', function() {
        if (document.getElementById('filter-dialog') === null) {
            var filterDialog = document.createElement('div');
            filterDialog.id = 'filter-dialog';
            filterDialog.innerHTML = '<div class="clear" title="Clear"></div><div class="close" title="Close"></div>' +
            '<table>' +
            '<tr><td><input type="checkbox" id="enable_alt"/></td><td class="filter-label">Altitude</td>     <td><input id="filter-alt-min" placeholder="from" /></td><td><input id="filter-alt-max" placeholder="to" /></td></tr>' +
            '<tr><td><input type="checkbox" id="enable_spd"/></td><td class="filter-label">Speed</td>        <td><input id="filter-spd-min" placeholder="from" /></td><td><input id="filter-spd-max" placeholder="to" /></td></tr>' +
            '<tr><td><input type="checkbox" id="enable_dis"/></td><td class="filter-label">Distance</td>     <td><input id="filter-dis-min" placeholder="from" /></td><td><input id="filter-dis-max" placeholder="to" /></td></tr>' +
            '<tr><td>&nbsp;</td></tr>' +
	    '<tr><td><input type="checkbox" id="fli_enable"/></td><td class="filter-label">Flight</td>       <td colspan="2"><input id="fli_filter"     placeholder="Flight ID w/ wildcard"            class="spanme" /></td></tr>' +
            '<tr><td><input type="checkbox" id="typ_enable"/></td><td class="filter-label">Aircraft Type</td><td colspan="2"><input id="typ_filter"     placeholder="A/C type w/ wildcard"            class="spanme" /></td></tr>' +
            '<tr><td><input type="checkbox" id="reg_enable"/></td><td class="filter-label">Fleet Watch</td>  <td colspan="2"><input id="reg_filter"     placeholder="ICAO & Registration w/ wildcard" class="spanme" /></td></tr>' +
            '<tr><td></td>                                        <td class="filter-label">Track Length</td> <td><input id="tracklength"    placeholder="sec"               /></td></tr>' +
            '<tr><td></td>                                        <td class="filter-label">Refresh Time</td> <td><input id="refreshtime"    placeholder="sec"               /></td></tr>' +
            '<tr><td></td>                                        <td class="filter-label"></td><td></td>    <td><button id="filter-apply">Apply &amp; Close</button>' +
            '</table>';
            controlDiv.appendChild(filterDialog);

            $("#enable_alt").prop( "checked", Filters.enable_alt );
            $("#enable_spd").prop( "checked", Filters.enable_spd );
            $("#enable_dis").prop( "checked", Filters.enable_dis );
            $("#reg_enable").prop( "checked", Filters.reg_enable );
            $("#typ_enable").prop( "checked", Filters.typ_enable );
            $("#fli_enable").prop( "checked", Filters.fli_enable );

            function handleChange() {
                Filters.altitude[0] = $("#filter-alt-min").val() || undefined;
                Filters.altitude[1] = $("#filter-alt-max").val() || undefined;
                Filters.enable_alt  = $("#enable_alt"    ).prop( "checked" );
                Filters.speed[0]    = $("#filter-spd-min").val() || undefined;
                Filters.speed[1]    = $("#filter-spd-max").val() || undefined;
                Filters.enable_spd  = $("#enable_spd"    ).prop( "checked" );
                Filters.distance[0] = $("#filter-dis-min").val() || undefined;
                Filters.distance[1] = $("#filter-dis-max").val() || undefined;
                Filters.enable_spd  = $("#enable_spd"    ).prop( "checked" );
                Filters.reg_filter  = $("#reg_filter"    ).val() || undefined;
                Filters.reg_enable  = $("#reg_enable"    ).prop( "checked" );
                Filters.tracklength = $("#tracklength"   ).val() || undefined;
                Filters.typ_enable  = $("#typ_enable"    ).prop( "checked" );
                Filters.typ_filter  = $("#typ_filter"    ).val() || undefined;
                Filters.fli_enable  = $("#fli_enable"    ).prop( "checked" );
                Filters.fli_filter  = $("#fli_filter"    ).val() || undefined;
                Filters.refreshtime = $("#refreshtime"   ).val() || undefined;
                setCookie("filters",JSON.stringify(Filters),9999);
                fetchData();
                
                console.log("handleChange refreshTime=",Filters.refreshtime);
            }

            $("#filter-alt-min, #filter-alt-max, #enable_alt, " +
              "#filter-spd-min, #filter-spd-max, #enable_spd, " +
              "#filter-dis-min, #filter-dis-max, #enable_dis, " +
              "#reg_filter, #reg_enable, " +
              "#typ_filter, #typ_enable, " +
              "#fli_filter, #fli_enable, " +
              "#tracklength, #refreshtime").change(handleChange);

            $("#filter-dialog .close").click(function() {
                // handleChange();
                $("#filter-dialog").hide();
            });

            $("#filter-dialog .clear").click(function() {
                $("#filter-alt-min, #filter-alt-max, " +
                  "#filter-spd-min, #filter-spd-max, " +
                  "#filter-dis-min, #filter-dis-max, " +
                  "#reg_filter, " +
                  "#typ_filter, " +
                  "#fli_filter").val('');
                $("#enable_alt, #enable_dis, #enable_spd, #reg_enable, #typ_enable, #fli_enable").prop( "checked", false );

                handleChange();
            });

            $("#filter-apply").click(function() {
                handleChange();
                $("#filter-dialog").hide();
            });
        }
        else
        {
            $("#filter-dialog").show();
        }

        $("#filter-alt-min").val(Filters.altitude[0]);
        $("#filter-alt-max").val(Filters.altitude[1]);
        $("#enable_alt"    ).val(Filters.enable_alt);
        $("#filter-spd-min").val(Filters.speed[0]);
        $("#filter-spd-max").val(Filters.speed[1]);
        $("#enable_spd"    ).val(Filters.enable_alt);
        $("#filter-dis-min").val(Filters.distance[0]);
        $("#filter-dis-max").val(Filters.distance[1]);
        $("#enable_dis"    ).val(Filters.enable_alt);
        $("#reg_filter"    ).val(Filters.reg_filter);
        $("#reg_enable"    ).val(Filters.reg_enable);
        $("#tracklength"   ).val(Filters.tracklength);
        $("#typ_enable"    ).val(Filters.typ_enable);
        $("#typ_filter"    ).val(Filters.typ_filter);
        $("#fli_enable"    ).val(Filters.fli_enable);
        $("#fli_filter"    ).val(Filters.fli_filter);
        $("#refreshtime"   ).val(Filters.refreshtime);

    });
}

function initialize()
{    
      /* There is no info_box_table without refreshGeneralInfo() */
      refreshGeneralInfo();

      map_box          = document.getElementById('map_box');
      map_info_box     = document.getElementById('map_info_box');
      flight_info_box  = document.getElementById('flight_info_box');
      info_box_table   = document.getElementById('info_box_table');
      hide_info        = document.getElementById('hide_info');
      hide_info_btn    = document.getElementById('hide_info_btn');

      document.getElementById("btn_hide_info_box").onclick = toggleInfoBox;

      /* INITIALIZE AC_SYMBOL_MAP
       * Aircraft symbol map is sorted in reverse order as otherwise
       * short plane names (e.g., C5) would be selected prior
       * to longer names (e.g., C550) and so the wrong icon
       * would be selected.
       */
      ac_symbol_map = ac_symbol_map.sort(sortPlaneSymbolList);

      Map = new google.maps.Map(map_box,{
        disableDefaultUI: true,
        center: new google.maps.LatLng(CenterLat, CenterLon),
        zoom: Zoomlevel,
        zoomControl: true,
        zoomControlOptions: {
                position: google.maps.ControlPosition.RIGHT_BOTTOM
        },
        scaleControl: false,
        streetViewControl: false
      });

      switch(MapStyle)
      {
        case "grey":
          Map.mapTypes.set('Grey map', rcmaptype1);
          Map.setMapTypeId('Grey map');
          break;
        case "terrain":
          Map.setMapTypeId(google.maps.MapTypeId.TERRAIN);
          break;
        case "roadmap":
          Map.setMapTypeId(google.maps.MapTypeId.ROADMAP);
          break;
        case "satellite":
          Map.setMapTypeId(google.maps.MapTypeId.SATELLITE);
          break;
        case "hybrid":
          Map.setMapTypeId(google.maps.MapTypeId.HYBRID);
          break;
        default:
          Map.setMapTypeId(google.maps.MapTypeId.TERRAIN);
      }

      /* Load position and zoom from Cookie */
      var currZoom = parseInt(getCookie("zoom"));
      if (currZoom > 0) Map.setZoom (currZoom);

      var currLat = getCookie("pos");
      var currLng = getCookie("poslng");
      var currCntr = new google.maps.LatLng (parseFloat(currLat),parseFloat(currLng));
      if (currLat > 0) Map.setCenter (currCntr);

      Map.addListener('click', deselectPlane);

      Map.addListener('zoom_changed', function () {
        setCookie("zoom",Map.getZoom(),9999)
      });

      Map.addListener('dragend', function () {
        var currCntr = new google.maps.LatLng;
        currCntr =  Map.getCenter().lat();
        setCookie("pos",currCntr,9999);
        currCntr =  Map.getCenter().lng();
        setCookie("poslng",currCntr,9999)
      });

      var newFilters = getCookie("filters");
      if (newFilters) {
          $.extend(Filters, JSON.parse(newFilters));
      }
      if (Filters.refreshtime == undefined) {
          Filters.refreshtime = 5;
      }

      var filterControlDiv = document.createElement('div');
      var filterControl = new FilterControl(filterControlDiv, Map);
      filterControlDiv.index = 1;
      Map.controls[google.maps.ControlPosition.RIGHT_BOTTOM].push(filterControlDiv);

      hideInfoBox();

      /* Setup night/day overlay */
      nite.init(Map);
      window.setInterval(function() {
        nite.setDate(null);
        nite.refresh();
      }, 20000); // every 20s
      
      var refreshLoop = function() {
          return function() {
              var timeout = Math.max (1000, Filters.refreshtime * 1000);
              //console.log("refreshLoop ", timeout);
              window.setTimeout(refreshLoop, timeout);
              fetchData();
              checkAircraftDeselected();
              refreshGeneralInfo();
          }
      }();

      window.setTimeout(refreshLoop, 0);

}
