/********************************************************************
 * 2D Map for Radarcape
 ********************************************************************
 * Guenter Koellner Embedded Development GmbH
 *
 * Am Rain 24
 * 85256 Vierkirchen
 * Geschaeftsfuehrer Guenter Koellner
 *
 * http://www.modesbeast.com
 ********************************************************************/

var aircraft_endpoint_prefix = ''
var mapinfo_endpoint_prefix = ''
if (is_mlat_srv === undefined) {
    var is_mlat_srv = false;
} else {
    var aircraft_endpoint_prefix = '/mlat'
    var mapinfo_endpoint_prefix = '/mlat'
}

var Gmap=null;
var Planes={};
var Filters={altitude:    [undefined, undefined],
             enable_alt:   undefined,
             speed:       [undefined, undefined],
             enable_spd:   undefined,
             distance:    [undefined, undefined],
             enable_dis:   undefined,
             typ_enable:   undefined,
             typ_filter:   undefined,
             fli_enable:   undefined,
             fli_filter:   undefined,
             dest_enable:  undefined,
             dest_filter:  undefined,
             orig_enable:  undefined,
             orig_filter:  undefined,
             fleetwatch:   undefined,
             reg_enable:   undefined,
             enable_adsb:  undefined,
             enable_mlat:  undefined,
             enable_flawa: undefined,
             enable_ogn:   undefined,
             enable_flarm: undefined,
             pref_source:  undefined,
             tracklength:  undefined,
             refreshtime:  undefined,
             mapstyle:     undefined
            };
var num_planesOnMap = 0;
var dbsize = 0;
var watched = 0;
var num_adsb = 0;
var num_mlat = 0;
var num_flawa = 0;
var num_ogn = 0;
var num_flarm = 0
var Selected=null;
var PreSelected=null;
var RangeCircle={};
var CircleStep=50;
var NauticalMiles_to_Meters=1852;
var KiloMeters_to_Meters=1000;

//var map_container;
var map_box;
var map_info_box;
var flight_info_box;
var hide_info;
var hide_info_btn;

var flightPath;

function getTimeString(isLocal) {
  var hours;
  var minutes;
  var date = new Date();
  var zone_offset = date.getTimezoneOffset();

  if(isLocal)
  {
    hours = date.getHours();
    minutes = date.getMinutes();
  }
  else
  {
    hours = date.getUTCHours();
    minutes = date.getUTCMinutes();
  }

  if(hours   < 10) {hours = '0'+hours;}
  if(minutes < 10) {minutes = '0'+minutes;}

  var zeit_string = hours + ':' + minutes;

  return zeit_string;
}

function getStrokeColor(alt) {
    var colorMap = [
      [    0,"ffffff"],
      [  100,"ffe062"],
      [  200,"ffea00"],
      [  300,"f0ff00"],
      [  400,"ccff00"],
      [  600,"42ff00"],
      [  800,"1eff00"],
      [ 1000,"00ff0c"],
      [ 1200,"00ff36"],
      [ 1500,"00ff72"],
      [ 2000,"00ff9c"],
      [ 2500,"00ffd2"],
      [ 3000,"00ffe4"],
      [ 3500,"00eaff"],
      [ 4000,"00c0ff"],
      [ 4500,"00a8ff"],
      [ 5000,"0096ff"],
      [ 5500,"0078ff"],
      [ 6000,"0054ff"],
      [ 6500,"0030ff"],
      [ 7000,"001eff"],
      [ 7500,"0000ff"],
      [ 8500,"1200ff"],
      [ 8500,"2400ff"],
      [ 9000,"3600ff"],
      [ 9500,"4e00ff"],
      [10000,"6000ff"],
      [10500,"7800ff"],
      [11000,"9600ff"],
      [11500,"ae00ff"],
      [12000,"d800ff"],
      [12500,"ff00e4"],
      [13000,"ff0000"]
    ];

    var color;
    for (var i = 0; i < colorMap.length; i++) {
      if (alt > colorMap[i][0]) {
        color = colorMap[i][1];
      } else {
        break;
      }
    }
    return color;
  }

function getIconForPlane(plane)
{

      var rFill = 255; gFill = 255; bFill = 0;
      var rBorder = 0, gBorder = 0, bBorder = 255;

      /* Colors for ADS-B/MLAT */
      if (plane.src == "M")
      {
         rBorder = 200; gBorder = 0; bBorder = 0;
      }
      else if (plane.src == "F") // FlightAware: green
      {
         rBorder = 000; gBorder = 100; bBorder = 0;
      }
      else if (plane.src == "L") // Flarm: dark magenta
      {
         rBorder = 0xff; gBorder = 0x40; bBorder = 0xff;
      }
      else if (plane.src == "O") // OGN: dark magenta
      {
         rBorder = 0xff; gBorder = 0x00; bBorder = 0xff;
      }
      else
      {
         rBorder = 64; gBorder = 64; bBorder = 64;
      }

      /* Change color for climb/decent */
      if (plane.vrt > 300)
      {
          /*rFill = 25; gFill = 165; bFill = 166;*/
          rFill = 53; gFill = 235; bFill = 244;
      }
      else if (plane.vrt < -300)
      {
          /* original A320: rFill = 148; gFill = 91; bFill = 1; */
          /*rFill = 228; gFill = 137; bFill = 0;*/
          rFill = 185; gFill = 92; bFill = 0;
      }

      if (plane.timeout == 1) /* after timeout 1 */
      {
          rFill = 128; gFill = 128; bFill = 128;  /* dark gray */
          rBorder = rFill; gBorder=gFill; bBorder=bFill;
      }

      var maxalt = 40000; /* Max altitude in the average case */
      var invalt = maxalt-plane.alt;
      var imageObj = get_image_object(plane.cat, plane.typ);

      var object_box = Raphael.pathBBox(imageObj.path);
      var offset_x = (object_box.x + object_box.x2) / 2;
      var offset_y = (object_box.y + object_box.y2) / 2;

      if (invalt < 0) invalt = 0;

      var strokew = 1;
      if (Selected == plane.hex) { strokew = 2; }
      if (plane.timeout == 2) { strokew = 0.333; }   /* light on  timeout 2 */

      var ac_symbol = {
         strokeColor: 'rgb('+rBorder+','+gBorder+','+bBorder+')',
         strokeWeight: strokew,
         path: imageObj.path,
         scale: imageObj.scale,
         fillColor: 'rgb('+rFill+','+gFill+','+bFill+')',
         fillOpacity: 0.9,
         anchor: new google.maps.Point(offset_x, offset_y),
         rotation: (plane.trk + imageObj.rotcorr) % 360
      };

      var myZoom = Gmap.getZoom();
      if((myZoom <= 14) && ((6 < myZoom))) {
         ac_symbol.scale = ac_symbol.scale + 0.2 * ((myZoom-10)/4);
      }
      else if(myZoom < 14) {
         ac_symbol.scale = ac_symbol.scale - 0.2
      }
      else {
         ac_symbol.scale = ac_symbol.scale + 0.2
      }

      return ac_symbol;
}


function showInfoBox()
{
    hide_info.style.visibility='visible';
    map_info_box.style.width='210px';
    hide_info_btn.setAttribute("class", "glyphicon glyphicon-minus");
}

function hideInfoBox()
{
    hide_info.style.visibility='hidden';
    map_info_box.style.width='1px';
    hide_info_btn.setAttribute("class", "glyphicon glyphicon-plus");
}

function toggleInfoBox()
{
  if(hide_info.style.visibility == 'hidden')
    showInfoBox();
  else
    hideInfoBox();
}

/********************************************************************************
 * clicking on a plane selects display of details in sidebar
 *******************************************************************************/
function selectPlane() {
    if (!Planes[this.planehex])
        return;

    var Old_Selected = Selected;
    Selected = this.planehex;

    if (Planes[Old_Selected]) {
        /* Remove the highlight in the previously selected plane. */
        Planes[Old_Selected].marker1.setIcon(getIconForPlane(Planes[Old_Selected]));
        labelSetStyle(Planes[Old_Selected], Planes[Old_Selected].marker1);
    }

    Planes[Selected].marker1.setIcon(getIconForPlane(Planes[Selected]));
    labelSetStyle(Planes[Selected], Planes[Selected].marker1);

    refreshSelectedInfo();
    fetchTrackData();

    showInfoBox();
}

function preselectPlane(plane) {
    PreSelected=null;
    console.log("preselectPlane: plane="+plane);

    if (!Planes[plane]) {
        console.log("preselectPlane: plane="+plane+" does not exist");
        return;
    }
    
    Selected=plane;
    Planes[Selected].marker1.setIcon(getIconForPlane(Planes[Selected]));
    labelSetStyle(Planes[Selected], Planes[Selected].marker1);

    var currCntr = new google.maps.LatLng(Planes[plane].lat, Planes[plane].lon);
    Gmap.setCenter(currCntr);

    refreshSelectedInfo();
    fetchTrackData();

    showInfoBox();
}

function updateURLPlaneSelection() {
    var hashObj = {};
    location.hash.replace('#', '').split('&').forEach(function(item) {
        hashObj[item.split('=')[0]] = item.split('=')[1];
    });
    PreSelected = hashObj.icao;
}

function deselectPlane() {
    if (Selected) {
        var Old_Selected = Selected;
        Selected = null;
        Planes[Old_Selected].marker1.setIcon(getIconForPlane(Planes[Old_Selected]));
        labelSetStyle(Planes[Old_Selected], Planes[Old_Selected].marker1);

        for ( var i = 0; i < flightPath.length; i++) {
            flightPath[i].setMap(null);
        }
        flightPath = [];

        flight_info_box.innerHTML = "";
    }
}


/*******************************************************************************
 * Check if an aircraft disappeared and therefore was deselected
 ******************************************************************************/
function checkAircraftDeselected() {
    if (Selected == null) {
        for ( var i = 0; i < flightPath.length; i++) {
            flightPath[i].setMap(null);
        }
        flightPath = [];
    }
}

/*******************************************************************************
 * Info box in div map_info_box - left side, always visible
 ******************************************************************************/
function refreshGeneralInfo()
{
  var html = '';

  html += "<table class=\"info_box_table\">";
  html += "  <thead>";

  html += "    <tr class=\"info_header\">";
  html += "      <td colspan=\"2\"><span class=\"glyphicon glyphicon-time\" aria-hidden=\"true\"></span> General Information</td>";
  html += "    </tr>";

  html += "  </thead>";
  html += "  <tbody>";

  html += "    <tr class=\"info_label\">";
  html += "      <td>UTC Time</td>";
  html += "      <td>Local Time</td>";
  html += "    </tr>";

  html += "    <tr class=\"info_value\">";
  html += "      <td>" + getTimeString(false) + "</td>";
  html += "      <td>" + getTimeString(true) + "</td>";
  html += "    </tr>";

  html += "  </tbody>";
  html += "  <tbody>";

  html += "    <tr class=\"info_label\" width=\"85%\">";
  html += "      <td>Totally received</td>";
  html += "      <td><b>" + dbsize +"</b></td>";
  html += "    </tr>";

  html += "    <tr class=\"info_label\">";
  html += "      <td>Currently watched</td>";
  html += "      <td><b>" + watched +"</b></td>";
  html += "    </tr>";

  html += "    <tr class=\"info_label\">";
  html += "      <td>Aircraft on map</td>";
  html += "      <td><b>" + num_planesOnMap +"</b></td>";
  html += "    </tr>";

  html += "    <tr class=\"info_label\">";
  html += "      <td>ADS-B locations</td>";
  html += "      <td><b>" + num_adsb +"</b></td>";
  html += "    </tr>";

  html += "    <tr class=\"info_label\">";
  html += "      <td>MLAT locations</td>";
  html += "      <td><b>" + num_mlat +"</b></td>";
  html += "    </tr>";

  html += "    <tr class=\"info_label\">";
  html += "      <td>FLARM locations</td>";
  html += "      <td><b>" + num_flarm +"</b></td>";
  html += "    </tr>";

  html += "  </tbody>";
  html += "</table>";

  document.getElementById('general_info_box').innerHTML = html;
}


/********************************************************************************
 * Additional info about plane, visible in left sidebar if a plane is selected
 *******************************************************************************/
function refreshSelectedInfo() {
  var p = undefined;

  if(Selected)
    p = Planes[Selected];

  if (p && Selected)
  {
    var html = '';

    html += '<table class="info_box_table">';

    /* AC Details Header */
    html += '    <tr class="info_header">';
    html += '      <td colspan="2"><span class="glyphicon glyphicon-plane" aria-hidden="true"></span> Aircraft Details</td>';
    html += '    </tr>';

    /* AC Details Row 1 */
    html += '  <tr class="info_label">';
    html += '    <td width="50%">Registration</td>';
    html += '    <td width="50%">Mode-S</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    if(p.reg != "") {
      html += '    <td><a href="http://www.planespotters.net/Aviation_Photos/search.php?tag='
           + encodeURIComponent(p.reg) + '" target="_blank">' + p.reg + '</a>&nbsp;</td>';
    } else {
      html += '    <td>' + p.reg + '&nbsp;</td>';
    }
    html += '    <td>'+ p.hex +'&nbsp;</td>';
    html += '  </tr>';

    /* AC Details Row 2 */
    html += '  <tr class="info_label">';
    html += '    <td>Type</td>';
    html += '    <td>Category</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.typ + '&nbsp;</td>';
    html += '    <td>' + p.cat + '&nbsp;</td>';
    html += '  </tr>';

    /* Flight Details Header */
    html += '    <tr class="info_header">';
    if (p.fli.length) {
      html += '    <td colspan="2"><span class="glyphicon glyphicon-plane" aria-hidden="true"></span> Flight '+ p.fli +'</td>';
    } else {
      html += '    <td colspan="2"><span class="glyphicon glyphicon-plane" aria-hidden="true"></span> Flight Details</td>';
    }
    html += '    </tr>';

    /* Flight Details Row 1 */
    html += '  <tr class="info_label">';
    html += '    <td>Origin</td>';
    html += '    <td>Destination</td>';
    html += '  </tr>';
    html += '  <tr class="info_value_normal">';
    html += '    <td><b>' + p.org + '&nbsp;</b><br/><small>' + resolveAirportById(p.org).label + '&nbsp;</small></td>';
    html += '    <td><b>' + p.dst + '&nbsp;</b><br/><small>' + resolveAirportById(p.dst).label + '&nbsp;</small></td>';
    html += '  </tr>';

    /* Flight Details Row 2 */
    html += '  <tr class="info_label">';
    html += '    <td>Track</td>';
    html += '    <td>Speed</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.trk + '&deg;</td>';
    html += '    <td>' + p.spd + ' kt</td>';
    html += '  </tr>';

    /* Flight Details Row 3 */
    html += '  <tr class="info_label">';
    html += '    <td>Altitude</td>';
    html += '    <td>Vertical Rate</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.alt + ' ft</td>';
    if(p.vrt) {
      html += '    <td>' + p.vrt + ' ft/min</td>';
    } else {
      html += '    <td></td>';
    }
    html += '  </tr>';

    /* AC Tracking Header */
    html += '    <tr class="info_header">';
    html += '      <td colspan="2"><span class="glyphicon glyphicon-map-marker" aria-hidden="true"></span> Aircraft Tracking</td>';
    html += '    </tr>';

    /* AC Tracking Row 1 */
    html += '  <tr class="info_label">';
    html += '    <td>Latitude</td>';
    html += '    <td>Longitude</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.lat + '&deg;</td>';
    html += '    <td>' + p.lon + '&deg;</td>';
    html += '  </tr>';

    /* AC Tracking Row 2 */
    html += '  <tr class="info_label">';
    html += '    <td>Distance</td>';
    html += '    <td>Source</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.dis.toFixed(1) + ' ' + DistanceUnit + '</td>';
    if(p.src == 'A') {
      html += '    <td>ADS-B</td>';
    }
    else if (p.src == 'M') {
      html += '    <td>MLAT</td>';
    }
    else if (p.src == 'F') {
      html += '    <td>FlAwa</td>';
    }
    else if (p.src == 'O') {
      html += '    <td>OGN</td>';
    }
    else if (p.src == 'L') {
      html += '    <td>Flarm</td>';
    }
    else {
      html += '    <td>&nbsp;</td>';
    }
    html += '  </tr>';

    /* AC Tracking Row 3 */
    html += '  <tr class="info_label">';
    html += '    <td>Last Signal</td>';
    html += '    <td>&nbsp;</td>';
    html += '  </tr>';
    html += '  <tr class="info_value">';
    html += '    <td>' + p.lla + ' sec</td>';
    html += '    <td>&nbsp;</td>';
    html += '  </tr>';

    html += '</table>';

    flight_info_box.innerHTML = html;
  }
  else
  {
    /* Remove Selected Info */
    flight_info_box.innerHTML = "<p>Click on an aircraft for more info.</p>";
  }
}


/********************************************************************************
 * Information that is displayed on map together with plane 
 *******************************************************************************/
function labelDataFlight(plane) {
    if (plane.fli == "") return "~" + plane.hex;
    return plane.fli;
}

function labelDataVerticalRate(plane) {
    var s = "";

    // Vertical Rate
    if (plane.vrt > -9000) {
        if (plane.vrt < -300) s =       Math.round(plane.vrt/100);
        if (plane.vrt >  300) s = "+" + Math.round(plane.vrt/100);
    }
    return s;
}

function labelDataAltitude(plane) {
    var s = "";

    // Aircraft on Ground
    if (plane.gda == "G")
    {
        s = "GND";
    }
    else
    {
        // Altitude
        s = Math.round(plane.alt/100);
        if (s < 0)
            s = "-0" + -1*s;
        else {
            if (s < 10)  s = "00" + s; else
            if (s < 100) s = "0" + s;
        }
    }
    return s;
}

function labelDataSpeed(plane) {
      return Math.round(plane.spd/10);
}

// This is the text that appears with all displayed aircraft
function labelDataTable(plane) {
    var labelDataTab = "<table class=\"aircraft_label\"><tr><td class=\"flight\" colspan=\"2\">" +
                       labelDataFlight(plane) +
                       "</td><td>" +
                       plane.src +
                       "</td></tr><tr><td class=\"altitude\">" +
                       labelDataAltitude(plane) +
                       "</td><td class=\"verticalRate\">" +
                       labelDataVerticalRate(plane) +
                       "</td><td class=\"speed\">" +
                       labelDataSpeed(plane) +
                       "</td></tr></table>";
    return labelDataTab;
}

function labelSetAnchor(plane, marker, offset) {
    if (plane.trk > 315) { marker.set("labelAnchor", new google.maps.Point( 66, -1 + offset)); return; }
    if (plane.trk < 45)  { marker.set("labelAnchor", new google.maps.Point( 66, -1 + offset)); return; }
    if (plane.trk < 135) { marker.set("labelAnchor", new google.maps.Point( 37, -1 + 28 + offset)); return; }
    if (plane.trk < 225) { marker.set("labelAnchor", new google.maps.Point(-17, -1 + offset)); return; }
    marker.set("labelAnchor", new google.maps.Point(37, -1 - 28 + offset));
}


function labelSetStyle(plane, marker) {
    if (plane.hex == Selected) {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'greenyellow'}); return; }
//    if (plane.squ == "7700")   {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orangered'}); return; }
//    if (plane.squ == "7600")   {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orangered'}); return; }
//    if (plane.squ == "7500")   {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orangered'}); return; }
//    if (plane.timeout == 1)    {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'silver'}); return; }
//    if (plane.timeout == 2)    {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'tan'}); return; }
//    if (plane.src == "M")      {marker.set("labelStyle", { opacity: 0.66, 'background-color': 'orange'}); return; }
    marker.set("labelStyle", {opacity: 0.6, 'background-color': 'gold'});
}


function fetchTrackData()
{
    if(Selected != null && Planes[Selected] != null) {
        var SelectedSource = Planes[Selected].src;
        var FlightPathURL = aircraft_endpoint_prefix + '/flightpath.json?icao=' + Selected + "&srcpref=" + SelectedSource;
  
        if ( typeof Filters.tracklength!=="undefined" && Filters.tracklength!==null )
        {
            FlightPathURL += "&lookback=" + Filters.tracklength
        }
  
        $.getJSON(FlightPathURL, function(data)
        {
            var trackpoints = new Array();
    
            for (var j=0; j < data[0].fpth.length; j++)
            {
                trackpoints.push( [new google.maps.LatLng(data[0].fpth[j].lat, data[0].fpth[j].lon), getStrokeColor(data[0].fpth[j].alt)] );
            }
    
            if(trackpoints.length > 0)
            {
                for (var i = 0; i < flightPath.length; i++) {
                    flightPath[i].setMap(null);
                }
                flightPath = [];
               
                for (var i = 1; i < trackpoints.length; i++) {
                    var pathSegment = new google.maps.Polyline({
                        path: [trackpoints[i-1][0], trackpoints[i][0]],
                        geodesic: true,
                        strokeColor: '#' + trackpoints[i][1],
                        strokeOpacity: 1.0,
                        strokeWeight: 2
                    });
                    pathSegment.setMap(Gmap);
                    flightPath.push(pathSegment);
                }
            }
        });
    }
}


var firstFetchData=true;
function fetchData() {

    if (Gmap.getBounds() == null) { // getBounds might not be defined even if map already is
        return;
    }

    var vps = Gmap.getBounds().getSouthWest().lat().toFixed(5);
    var vpw = Gmap.getBounds().getSouthWest().lng().toFixed(5);

    var vpn = Gmap.getBounds().getNorthEast().lat().toFixed(5);
    var vpe = Gmap.getBounds().getNorthEast().lng().toFixed(5);
    if( firstFetchData ) {
        firstFetchData=false;
        viewpoint = "";
    } else {
        var viewpoint = '&vpn=' + vpn + '&vpe=' + vpe + '&vps=' + vps + '&vpw=' + vpw
    }

    var latcntr = Gmap.getCenter().lat().toFixed(5);
    var loncntr = Gmap.getCenter().lng().toFixed(5);

    var alt_min = "";
    var alt_max = "";
    if (Filters.enable_alt) {
        if (typeof Filters.altitude[0] !== "undefined" && Filters.altitude[0] !== null) {
            alt_min = "&altitude.from=" + Filters.altitude[0];
        }

        if (typeof Filters.altitude[1] !== "undefined" && Filters.altitude[1] !== null) {
            alt_max = "&altitude.to=" + Filters.altitude[1];
        }
    }

    var spd_min = "";
    var spd_max = "";
    if (Filters.enable_spd) {
        if (typeof Filters.speed[0] !== "undefined" && Filters.speed[0] !== null) {
            spd_min = "&speed.from=" + Filters.speed[0];
        }
        if (typeof Filters.speed[1] !== "undefined" && Filters.speed[1] !== null) {
            spd_max = "&speed.to=" + Filters.speed[1];
        }
    }

    var dis_min = "";
    var dis_max = "";
    if (Filters.enable_dis) {
        if (typeof Filters.distance[0] !== "undefined" && Filters.distance[0] !== null) {
            var dis_min = "&distance.from=" + Filters.distance[0];
        }

        if (typeof Filters.distance[1] !== "undefined" && Filters.distance[1] !== null) {
            var dis_max = "&distance.to=" + Filters.distance[1];
        }
    }

    var fleetwatch = "";
    if (Filters.reg_enable && typeof Filters.reg_filter !== "undefined" && Filters.reg_filter !== null) {
        fleetwatch = "&icao=" + Filters.reg_filter;
    }

    var ac_type = "";
    if (Filters.typ_enable && typeof Filters.typ_filter !== "undefined" && Filters.typ_filter !== null) {
        ac_type = "&type=" + Filters.typ_filter;
    }

    var flight = "";
    if (Filters.fli_enable && typeof Filters.fli_filter !== "undefined" && Filters.fli_filter !== null) {
        flight = "&flight=" + Filters.fli_filter;
    }

    var dest = "";
    if (Filters.dest_enable && typeof Filters.dest_filter !== "undefined" && Filters.dest_filter !== null) {
        dest = "&dest=" + Filters.dest_filter;
    }

    var orig = "";
    if (Filters.orig_enable && typeof Filters.orig_filter !== "undefined" && Filters.orig_filter !== null) {
        orig = "&orig=" + Filters.orig_filter;
    }

    // C++ assumes that default is all on, so we only need to send the 'off'
    // condition
    var srcexcl = "";
    if (!Filters.enable_adsb) {
        srcexcl = srcexcl + "&srcexcl=a";
    }
    if (!Filters.enable_mlat) {
        srcexcl = srcexcl + "&srcexcl=m";
    }
    if (!Filters.enable_flawa) {
        srcexcl = srcexcl + "&srcexcl=f";
    }
    if (!Filters.enable_ogn) {
        srcexcl = srcexcl + "&srcexcl=o";
    }
    if (!Filters.enable_flarm) {
        srcexcl = srcexcl + "&srcexcl=l";
    }

    var srcpref = "";
    if (typeof Filters.pref_source !== "undefined" && Filters.pref_source !== null && Filters.perfsource !== "") {
        srcpref = "&srcpref=" + Filters.pref_source;
    }

    $.getJSON(aircraft_endpoint_prefix + '/aircraftlist.json?knownposonly=1' 
              + viewpoint + fleetwatch + alt_min + alt_max + spd_min + spd_max + dis_min + dis_max + ac_type + flight + dest + orig + srcexcl + srcpref, 
              function(data) 
    {
        var stillhere = {}
        for ( var j = 0; j < data.length; j++) {
            var plane = data[j];
            var marker1 = null;

            stillhere[plane.hex] = true;
            plane.fli = $.trim(plane.fli);

            /* update */
            if (Planes[plane.hex]) {
                marker1 = Planes[plane.hex].marker1;

                var newpos = new google.maps.LatLng(plane.lat, plane.lon);

                /* update flight number */
                marker1.setIcon(getIconForPlane(plane));
                marker1.setPosition(newpos);
                marker1.set("labelContent", labelDataTable(plane));
                labelSetAnchor(plane, marker1, 12);
                labelSetStyle(plane, marker1);
                marker1.label.draw();

                if (plane.hex == Selected)
                    refreshSelectedInfo();

            } else {
                /* new flight */
                marker1 = new MarkerWithLabel({
                    position : new google.maps.LatLng(plane.lat, plane.lon),
                    draggable : false,
                    raiseOnDrag : false,
                    zIndex : 70000,
                    map : Gmap,
                    labelContent : labelDataTable(plane),
                    labelAnchor : new google.maps.Point(0, 0),
                    labelClass : "labels",
                    labelStyle : {},
                    labelZIndex : 60000,
                    icon : getIconForPlane(plane)
                });

                labelSetAnchor(plane, marker1, 12);
                labelSetStyle(plane, marker1);

                /* Trap clicks for this marker. */
                marker1.addListener('click', selectPlane);
            }

            marker1.planehex = plane.hex; /* for click */
            plane.marker1 = marker1;
            Planes[plane.hex] = plane;

            if (plane.fli.length == 0) {
                marker1.setTitle(plane.hex)
            } else {
                /* Popup window when mouse over aircraft icon */
                var title = plane.reg + ' (' + plane.hex + ')';
                if (plane.org.length && plane.dst.length)
                    title += '\n' + plane.org + '>' + plane.dst;
                marker1.setTitle(title)
            }
        }
        num_planesOnMap = data.length;

        /* Remove idle planes. */
        for ( var p in Planes) {
            if (!stillhere[Planes[p].hex]) {
                if (Planes[p].hex == Selected) {
                    deselectPlane();
                }
                Planes[p].marker1.setMap(null);
                delete Planes[p];
            }
        }
        
        if( PreSelected!=null ) {
            console.log("PreSelected="+PreSelected);
            preselectPlane(PreSelected);
            PreSelected=null;
        }

    });

    $.getJSON(mapinfo_endpoint_prefix + '/mapinfo.json', function(data) {
        dbsize = data.dbsize;
        watched = data.watched;
        num_adsb = data.num_adsb;
        num_mlat = data.num_mlat;
        num_flawa = data.num_flawa;
        num_ogn = data.num_ogn;
        num_flarm = data.num_flarm
    });
}

function FilterControl(controlDiv)
{
    var controlUI = document.createElement('div');
    controlUI.className = 'filter-button';
    controlUI.title = 'Click to modify the filters';
    controlDiv.appendChild(controlUI);

    var controlText = document.createElement('div');
    controlText.className = 'filter-icon';
    controlUI.appendChild(controlText);

    controlUI.addEventListener('click', function() {
        if (document.getElementById('filter-dialog') === null) {
            var filterDialog = document.createElement('div');
            filterDialog.id = 'filter-dialog';
            filterDialog.innerHTML = '<div class="clear" title="Clear"></div><div class="close" title="Close"></div>' +
            '<table>' +
            '<tr><td><input type="checkbox" id="enable_alt"/></td><td class="filter-label">Altitude         </td><td><input id="filter-alt-min" placeholder="from" /></td><td><input id="filter-alt-max" placeholder="to"/></td></tr>' +
            '<tr><td><input type="checkbox" id="enable_spd"/></td><td class="filter-label">Speed            </td><td><input id="filter-spd-min" placeholder="from" /></td><td><input id="filter-spd-max" placeholder="to"/></td></tr>' +
            '<tr><td><input type="checkbox" id="enable_dis"/></td><td class="filter-label">Distance         </td><td><input id="filter-dis-min" placeholder="from" /></td><td><input id="filter-dis-max" placeholder="to"/></td></tr>' +
            '<tr><td></td></tr>' +
            '<tr><td><input type="checkbox" id="dest_enable"/></td><td class="filter-label">Destination     </td><td colspan="2"><input id="dest_filter"    placeholder="Flight Destination w/ wildcard"  class="spanme" /></td></tr>' +
            '<tr><td><input type="checkbox" id="orig_enable"/></td><td class="filter-label">Origin          </td><td colspan="2"><input id="orig_filter"    placeholder="Flight Origin w/ wildcard"       class="spanme" /></td></tr>' +
            '<tr><td><input type="checkbox" id="fli_enable"/> </td><td class="filter-label">Flight          </td><td colspan="2"><input id="fli_filter"     placeholder="Flight ID w/ wildcard"           class="spanme" /></td></tr>' +
            '<tr><td><input type="checkbox" id="typ_enable"/> </td><td class="filter-label">Aircraft Type   </td><td colspan="2"><input id="typ_filter"     placeholder="A/C type w/ wildcard"            class="spanme" /></td></tr>' +
            '<tr><td><input type="checkbox" id="reg_enable"/> </td><td class="filter-label">Fleet Watch     </td><td colspan="2"><input id="reg_filter"     placeholder="6 digit ICAO or Registration w/ wildcard" class="spanme" /></td></tr>' +
            '<tr><td>                                         </td><td>                                     </td><td class="filter-label" colspan="2"><input type="checkbox" id="enable_adsb"/> ADSB <input type="checkbox" id="enable_mlat"/> MLAT <input type="checkbox" id="enable_flawa"/> FlaWa <input type="checkbox" id="enable_ogn"/> OGN <input type="checkbox" id="enable_flarm"/> Flarm</td></tr>' +
            '<tr><td>                                         </td><td class="filter-label">Preferred Source</td>' +
            '<td><select id="pref_source"><option value="">Auto</option><option value="A">ADS-B</option><option value="M">Jetvision MLAT</option><option value="F">FlightAware</option><option value="O">Open Glider Net Server</option><option value="L">OGN Local Receiver</option></select></td></tr>' +
            '<tr><td>                                         </td><td class="filter-label">Track Length    </td><td><input id="tracklength"     placeholder="sec"               /></td></tr>' +
            '<tr><td>                                         </td><td class="filter-label">Refresh Time    </td><td><input id="refreshtime"     placeholder="sec"               /></td>' +
            '<td>Map Style <select id="mapstyle"><option value="grey">Grey</option><option value="terrain">Terrain</option><option value="roadmap">Roadmap</option><option value="satellite">Satellite</option><option value="hybrid">Hybrid</option></select></td></tr>' +
            '<tr><td>                                         </td><td class="filter-label">                </td><td>                                                              </td>    <td><button id="filter-apply">Apply &amp; Close</button>' +
            '</table>';
            controlDiv.appendChild(filterDialog);


            $("#enable_alt" ).prop ( "checked", Filters.enable_alt   );
            $("#enable_spd" ).prop ( "checked", Filters.enable_spd   );
            $("#enable_dis" ).prop ( "checked", Filters.enable_dis   );
            $("#reg_enable" ).prop ( "checked", Filters.reg_enable   );
            $("#typ_enable" ).prop ( "checked", Filters.typ_enable   );
            $("#fli_enable" ).prop ( "checked", Filters.fli_enable   );
            $("#dest_enable").prop ( "checked", Filters.dest_enable  );
            $("#orig_enable").prop ( "checked", Filters.orig_enable  );
            $("#enable_adsb").prop ( "checked", Filters.enable_adsb  );
            $("#enable_mlat").prop ( "checked", Filters.enable_mlat  );
            $("#enable_flawa").prop( "checked", Filters.enable_flawa );
            $("#enable_ogn"  ).prop( "checked", Filters.enable_ogn   );
            $("#enable_flarm").prop( "checked", Filters.enable_flarm );

            function handleChange() {
                Filters.altitude[0] = $("#filter-alt-min").val() || undefined;
                Filters.altitude[1] = $("#filter-alt-max").val() || undefined;
                Filters.enable_alt  = $("#enable_alt"    ).prop( "checked" );
                Filters.speed[0]    = $("#filter-spd-min").val() || undefined;
                Filters.speed[1]    = $("#filter-spd-max").val() || undefined;
                Filters.enable_spd  = $("#enable_spd"    ).prop( "checked" );
                Filters.distance[0] = $("#filter-dis-min").val() || undefined;
                Filters.distance[1] = $("#filter-dis-max").val() || undefined;
                Filters.enable_dis  = $("#enable_dis"    ).prop( "checked" );
                Filters.reg_filter  = $("#reg_filter"    ).val() || undefined;
                Filters.reg_enable  = $("#reg_enable"    ).prop( "checked" );
                Filters.tracklength = $("#tracklength"   ).val() || undefined;
                Filters.typ_enable  = $("#typ_enable"    ).prop( "checked" );
                Filters.typ_filter  = $("#typ_filter"    ).val() || undefined;
                Filters.fli_enable  = $("#fli_enable"    ).prop( "checked" );
                Filters.fli_filter  = $("#fli_filter"    ).val() || undefined;
                Filters.dest_enable = $("#dest_enable"   ).prop( "checked" );
                Filters.dest_filter = $("#dest_filter"   ).val() || undefined;
                Filters.orig_enable = $("#orig_enable"   ).prop( "checked" );
                Filters.orig_filter = $("#orig_filter"   ).val() || undefined;
                Filters.refreshtime = $("#refreshtime"   ).val() || undefined;
                Filters.enable_adsb = $("#enable_adsb "  ).prop( "checked" );
                Filters.enable_mlat = $("#enable_mlat "  ).prop( "checked" );
                Filters.enable_flawa= $("#enable_flawa"  ).prop( "checked" );
                Filters.enable_ogn  = $("#enable_ogn  "  ).prop( "checked" );
                Filters.enable_flarm= $("#enable_flarm"  ).prop( "checked" );
                Filters.pref_source = $("#pref_source"   ).val() || undefined;
                Filters.mapstyle    = $("#mapstyle"      ).val() || undefined;

                setCookie("filters",JSON.stringify(Filters),9999);
                fetchData();
            }

            $("#filter-alt-min, #filter-alt-max, #enable_alt, " +
              "#filter-spd-min, #filter-spd-max, #enable_spd, " +
              "#filter-dis-min, #filter-dis-max, #enable_dis, " +
              "#reg_filter, #reg_enable, " +
              "#typ_filter, #typ_enable, " +
              "#fli_filter, #fli_enable, " +
              "#dest_filter, #dest_enable, " +
              "#orig_filter, #orig_enable, " +
              "#tracklength, #refreshtime, " +
              "#pref_source, #mapstyle, " +
              "#enable_adsb, #enable_mlat, #enable_flawa, #enable_ogn, #enable_flarm"
              ).change(handleChange);

            $("#filter-dialog .close").click(function() {
                // handleChange();
                $("#filter-dialog").hide();
            });

            $("#filter-dialog .clear").click(function() {
                $("#filter-alt-min, #filter-alt-max, " +
                  "#filter-spd-min, #filter-spd-max, " +
                  "#filter-dis-min, #filter-dis-max, " +
                  "#reg_filter, " +
                  "#typ_filter, " +
                  "#fli_filter, " +
                  "#dest_filter, " +
                  "#orig_filter, #pref_source").val(null);
                $("#enable_alt, #enable_dis, #enable_spd, #reg_enable, #typ_enable, #fli_enable, #dest_enable, #orig_enable").prop( "checked", false );
                $("#enable_adsb, #enable_mlat, #enable_flawa, #enable_ogn, #enable_flarm"                                   ).prop( "checked", true );

                handleChange();
            });

            $("#filter-apply").click(function() {
                handleChange();
                $("#filter-dialog").hide();
            });
        }
        else
        {
            $("#filter-dialog").show();
        }

        $("#filter-alt-min").val(Filters.altitude[0]);
        $("#filter-alt-max").val(Filters.altitude[1]);
        $("#enable_alt"    ).val(Filters.enable_alt);
        $("#filter-spd-min").val(Filters.speed[0]);
        $("#filter-spd-max").val(Filters.speed[1]);
        $("#enable_spd"    ).val(Filters.enable_alt);
        $("#filter-dis-min").val(Filters.distance[0]);
        $("#filter-dis-max").val(Filters.distance[1]);
        $("#enable_dis"    ).val(Filters.enable_alt);
        $("#reg_filter"    ).val(Filters.reg_filter);
        $("#reg_enable"    ).val(Filters.reg_enable);
        $("#tracklength"   ).val(Filters.tracklength);
        $("#typ_enable"    ).val(Filters.typ_enable);
        $("#typ_filter"    ).val(Filters.typ_filter);
        $("#fli_enable"    ).val(Filters.fli_enable);
        $("#fli_filter"    ).val(Filters.fli_filter);
        $("#dest_enable"   ).val(Filters.dest_enable);
        $("#dest_filter"   ).val(Filters.dest_filter);
        $("#orig_enable"   ).val(Filters.orig_enable);
        $("#orig_filter"   ).val(Filters.orig_filter);
        $("#refreshtime"   ).val(Filters.refreshtime);
        $("#mapstyle"      ).val(Filters.mapstyle);
        $("#enable_adsb"   ).val(Filters.enable_adsb);
        $("#enable_mlat"   ).val(Filters.enable_mlat);
        $("#enable_flawa"  ).val(Filters.enable_flawa);
        $("#enable_ogn"    ).val(Filters.enable_ogn);
        $("#enable_flarm"  ).val(Filters.enable_flarm);
        $("#pref_source"   ).val(Filters.pref_source);
    });
}


function initialize() {
    
    updateURLPlaneSelection();

    /* There is no info_box_table without refreshGeneralInfo() */
    refreshGeneralInfo();

    map_box = document.getElementById('map_box');
    map_info_box = document.getElementById('map_info_box');
    flight_info_box = document.getElementById('flight_info_box');
    info_box_table = document.getElementById('info_box_table');
    hide_info = document.getElementById('hide_info');
    hide_info_btn = document.getElementById('hide_info_btn');

    document.getElementById("btn_hide_info_box").onclick = toggleInfoBox;

    Gmap = new google.maps.Map(map_box, {
        disableDefaultUI : true,
        center : new google.maps.LatLng(CenterLat, CenterLon),
        zoom : Zoomlevel,
        zoomControl : true,
        zoomControlOptions : {
            position : google.maps.ControlPosition.RIGHT_BOTTOM
        },
        scaleControl : false,
        streetViewControl : false
    });

    var newFilters = getCookie("filters");
    if (newFilters) {
        $.extend(Filters, JSON.parse(newFilters));
    }

    if (Filters.enable_adsb == "undefined" || Filters.enable_adsb == null) {
        Filters.enable_adsb = true;
    }
    if (Filters.enable_mlat == "undefined" || Filters.enable_mlat == null) {
        Filters.enable_mlat = true;
    }
    if (Filters.enable_flawa == "undefined" || Filters.enable_flawa == null) {
        Filters.enable_flawa = true;
    }
    if (Filters.enable_ogn == "undefined" || Filters.enable_ogn == null) {
        Filters.enable_ogn = true;
    }
    if (Filters.enable_flarm == "undefined" || Filters.enable_flarm == null) {
        Filters.enable_flarm = true;
    }

    switch (Filters.mapstyle) {
        case "grey":
            Gmap.mapTypes.set('Grey map', rcmaptype1);
            Gmap.setMapTypeId('Grey map');
            break;
        case "terrain":
            Gmap.setMapTypeId(google.maps.MapTypeId.TERRAIN);
            break;
        case "roadmap":
            Gmap.setMapTypeId(google.maps.MapTypeId.ROADMAP);
            break;
        case "satellite":
            Gmap.setMapTypeId(google.maps.MapTypeId.SATELLITE);
            break;
        case "hybrid":
            Gmap.setMapTypeId(google.maps.MapTypeId.HYBRID);
            break;
        default:
            Gmap.setMapTypeId(google.maps.MapTypeId.TERRAIN);
    }

    /* Load position and zoom from Cookie */
    var currZoom = parseInt(getCookie("zoom"));
    if (currZoom > 0)
        Gmap.setZoom(currZoom);

    var currLat = getCookie("pos");
    var currLng = getCookie("poslng");
    var currCntr = new google.maps.LatLng(parseFloat(currLat), parseFloat(currLng));
    if (currLat > 0)
        Gmap.setCenter(currCntr);

    flightPath = [];

    Gmap.addListener('click', deselectPlane);

    Gmap.addListener('zoom_changed', function() {
        setCookie("zoom", Gmap.getZoom(), 9999)
    });

    Gmap.addListener('dragend', function() {
        var currCntr = new google.maps.LatLng;
        currCntr = Gmap.getCenter().lat();
        setCookie("pos", currCntr, 9999);
        currCntr = Gmap.getCenter().lng();
        setCookie("poslng", currCntr, 9999)
    });

    loadAirports();

    var filterControlDiv = document.createElement('div');
    var filterControl = new FilterControl(filterControlDiv);
    filterControlDiv.index = 1;
    Gmap.controls[google.maps.ControlPosition.RIGHT_BOTTOM].push(filterControlDiv);

    hideInfoBox();

    /* Setup night/day overlay */
    nite.init(Gmap);
    window.setInterval(function() { nite.setDate(null); nite.refresh(); }, 20000); // every 20s

    var refreshLoop = function() {
        return function() {
            if (typeof Filters.refreshtime !== "number") {
                var timeout = 1000;
            } else {
                var timeout = Math.max(1000, 1000 * refreshtime);
            }
            window.setTimeout(refreshLoop, timeout);
            fetchData();
            fetchTrackData();
            checkAircraftDeselected();
            refreshGeneralInfo();
        }
    }();

    window.setTimeout(refreshLoop, 0);

}
