/********************************************************************
* Dynamic Receiver Statistic for Radarcape
********************************************************************
* Guenter Koellner Embedded Development GmbH
*
* Am Rain 24
* 85256 Vierkirchen
* Geschaeftsfuehrer Guenter Koellner
*
* References:
* https://www.w3schools.com/jsref/default.asp
* http://www.chartjs.org/docs/latest/
* http://www.chartjs.org/samples/latest/charts/radar.html
*
* http://www.modesbeast.com
********************************************************************/

var overallChart;
var singleChart = [];
var distunit = "";
var refresh_time = 5; // seconds

function expand360 ( labels, values ) {
    var myLabels = []
    var myValues = []
    
    var index = 0;
    for (var i=0; i<360; i++) {
        myLabels.push(i);
        if( labels[index]==i ) {
            myValues.push(values[index]);
            index++;
        }
        else {
            myValues.push(0);
        }
    }
    values = myValues;
    labels = myLabels;
}


function updateCharts() {
    $.getJSON( "/range.json", function(json_data) {
        for (var i=0; i<json_data["ranges"].length; i++) {
            
            document.getElementById('distunit').innerHTML= "Unit: "+json_data["distunit"]+"<p/>";   
            refresh_time = ( typeof json_data["refreshstatus"]!=='undefined' ) ? json_data["refreshstatus"] : refresh_time; 
            
            var labels = json_data["ranges"][i]["azimut"];
            var values = json_data["ranges"][i]["range"];
            
            var labels360 = [];
            var values360 = [];
            var index = 0;
            for (var j=0; j<360; j++) {
                if( j%15 == 0) {
                    labels360.push(j);
                } else {
                    labels360.push("");
                }
                if( labels[index]==j ) {
                    values360.push(values[index]);
                    index++;
                }
                else {
                    values360.push(0);
                }
            }
            
            overallChart.data.labels           = labels360; 
            overallChart.data.datasets[i].data = values360;
            
            switch (i) {
                case 0:
                    singleChart[0].data.labels           = labels360;
                    singleChart[0].data.datasets[0].data = values360;
                    singleChart[0].update();       
                    break;
                case 1:
                    singleChart[1].data.labels           = labels360;
                    singleChart[1].data.datasets[0].data = values360;
                    singleChart[1].update();       
                    break;
                case 2:
                    singleChart[2].data.labels           = labels360;
                    singleChart[2].data.datasets[0].data = values360;
                    singleChart[2].update();       
                    break;
            }
        }
        overallChart.update();
    });
}

$(document).ready(function() {

    // https://stackoverflow.com/questions/43010404/chart-js-radar-scaleshowline-for-each-scaleline
    
    var color = Chart.helpers.color;
    
    var options10 = {
            responsive: true, // true: scales with responsiveness, false: scales with pixel size
            animation: false,
            legend: {
                position: 'bottom',
            },
            scale: {
                // https://www.chartjs.org/docs/latest/axes/radial/linear.html#angle-line-options
                // angleLines: { display: true, color: "#00ff00ff" },
                // https://www.chartjs.org/docs/latest/axes/styling.html
                // display: true,
                ticks: { min:0, stepSize: 10 } // this is the maximum distance to be shown
            }
        }

    var options25 = {
            responsive: true, // true: scales with responsiveness, false: scales with pixel size
            animation: false,
            legend: {
                position: 'bottom',
            },
            scale: {
                // https://www.chartjs.org/docs/latest/axes/radial/linear.html#angle-line-options
                // https://www.chartjs.org/docs/latest/axes/radial/linear.html#linear-radial-axis
                // angleLines: { display: true, color: "#00ff00ff", lineWidth: 0.1 },
                // https://www.chartjs.org/docs/latest/axes/styling.html
                // display: true,
                ticks: { min:0, 
                         stepSize: 25,
                         callback: function(value, index, values) {
                             //console.log (value, index, values);
                             return value;
                         }
                } // this is the maximum distance to be shown
            }
        }

    var dataset0a = {
            label: '3000ft',
            fill: false,
            borderColor: "#ff0000ff",
            //pointBackgroundColor: "#ff0000ff",
            borderWidth: 2,
            pointRadius: 0
        };
    var dataset1a = {
            label: '20000ft',
            fill: false,
            borderColor: "#0000ffff",
            //pointBackgroundColor: "#0000ffff",
            borderWidth: 2,
            pointRadius: 0
        };
    var dataset2a = {
            label: 'Max. Altitude',
            fill: false,
            borderColor: "#000000ff", //window.chartColors.black,
            //pointBackgroundColor: "#000000ff", //window.chartColors.black
            borderWidth: 2,
            pointRadius: 0
        };

    var datasets = [dataset0a, dataset1a, dataset2a];
    overallChart = new Chart(document.getElementById('range-overall'), { type:'radar', data:{datasets}, options:options25 } );

    var dataset0b = {
            label: '3000ft',
            fill: false,
            borderColor: "#ff0000ff",
            //pointBackgroundColor: "#ff0000ff",
            borderWidth: 2,
            pointRadius: 0
        };

    var datasets = [dataset0b];

    singleChart[0] = new Chart(document.getElementById('range-0'), { type:'radar', data:{datasets}, options:options10 } );

    var dataset1c = {
            label: '20000ft',
            fill: false,
            borderColor: "#0000ffff",
            //pointBackgroundColor: "#0000ffff",
            borderWidth: 2,
            pointRadius: 0
        };

    datasets = [dataset1c];

    singleChart[1] = new Chart(document.getElementById('range-1'), { type:'radar', data:{datasets}, options:options25 } );


    var dataset2d = {
            label: 'Max. Altitude',
            fill: false,
            borderColor: "#000000ff", //window.chartColors.black,
            //pointBackgroundColor: "#000000ff", //window.chartColors.black
            borderWidth: 2,
            pointRadius: 0
        };

    datasets = [dataset2d];

    singleChart[2] = new Chart(document.getElementById('range-2'), { type:'radar', data:{datasets}, options:options25 } );

    function loop() {
        updateCharts();
        var delay = Math.max(1000, refresh_time*1000);
        setTimeout( loop, delay );
      }

    loop();
});

