/********************************************************************
 * Dynamic Receiver Statistic for Radarcape
 ********************************************************************
 * Guenter Koellner Embedded Development GmbH
 *
 * Am Rain 24
 * 85256 Vierkirchen
 * Geschaeftsfuehrer Guenter Koellner
 *
 * References:
 * https://www.w3schools.com/jsref/default.asp
 * http://www.chartjs.org/docs/latest/
 *
 * http://www.modesbeast.com
 ********************************************************************/

var	myChartNofGnssSv;
var refresh_time = 1;

function get_date_time_string (unix_timestamp, nanoseconds) {
    // Create a new JavaScript Date object based on the timestamp
    // multiplied by 1000 so that the argument is in milliseconds, not seconds.
    var date = new Date(unix_timestamp*1000);
    // Hours part from the timestamp
    var hours = date.getHours();
    var minutes = "0" + date.getMinutes();
    var seconds = "0" + date.getSeconds();
    var formattedTime = date.toISOString();
    formattedTime = formattedTime.replace("T"," ");
    formattedTime = formattedTime.substr(0, 19);

    return formattedTime;
}

function get_time_string (unix_timestamp, nanoseconds) {
    // Create a new JavaScript Date object based on the timestamp
    // multiplied by 1000 so that the argument is in milliseconds, not seconds.
    var date = new Date(unix_timestamp*1000);
    // Hours part from the timestamp
    var formattedTime = date.toTimeString();
    formattedTime = formattedTime.substr(0, 8);

    return formattedTime;
}


function updateTable() {
    $.getJSON( "/gps.json", function(data) {

        refresh_time = ( typeof data["refreshstatus"]!=='undefined' ) ? data["refreshstatus"] : refresh_time; 

        var table_body = document.getElementById("receiver_status").tBodies[0];
    
        var receiver_status = data["ReceiverStatus"];
        var keys = Object.keys(receiver_status);
        for( var i=0; i< keys.length; i++ ) {
        
            var rows_len = table_body.rows.length; //first is row to clone
            if( rows_len-1<i ) {
                var row = table_body.insertRow(i);
                var cell1 = row.insertCell(0);
                cell1.innerHTML="<b>"+keys[i]+"</b>";
                var cell2 = row.insertCell(1);
            }
            else
            {
                var cell2 = table_body.rows[i].cells[1];
            }
                
            cell2.innerHTML=receiver_status[keys[i]];
        }
        
        // --------------------------------------
        
        var sat_table = document.getElementById("sat_table");
        
        var sat_table_header = sat_table.tHead;
        if ( sat_table_header==null ) {
            
            sat_table_header = sat_table.createTHead();
            var row = sat_table_header.insertRow(0);
            
            var SatTableHeader = data["SatTable"]["Header"]
            for( var i=0; i<SatTableHeader.length; i++ ) {
                row.insertCell(i).outerHTML = "<th>"+SatTableHeader[i]+"</th>";
            }
        }

        var sat_table_body = sat_table.tBodies[0];
        var sat_table_values = data["SatTable"]["Values"]

        while ( sat_table_body.rows.length > sat_table_values.length ) {
            sat_table_body.deleteRow(-1);
        }


        for( var i=0; i<sat_table_values.length; i++ ) {
        
            var rows_len = sat_table_body.rows.length;
            if( rows_len-1<i ) {
                rows_len++;
                var row = sat_table_body.insertRow(i);
                for( var j=0; j<sat_table_values[i].length; j++ ) {
                    cell = row.insertCell(j);
                    cell.style.textAlign = 'center';
                }
            }

            for( var j=0; j<sat_table_values[i].length; j++ ) {
                var cell = sat_table_body.rows[i].cells[j];
                cell.innerHTML=sat_table_values[i][j];
            }
        }


        // --------------------------------------
        
        var err_log = document.getElementById("error_log");
        
        error_log.innerHTML=data["ErrorLog"];
        
        // --------------------------------------
        
        var sat_visibility = data["SatVisibility"]
        var graphs = sat_visibility.graphs;
                
        var xAxis_labels = [];
        for (var i = 0; i < graphs[0].values.length; i++) {
            xAxis_labels.push(get_time_string(sat_visibility["time"]+i*sat_visibility["resolution"]))
        }

        myChartNofGnssSv.data.labels = xAxis_labels;
        for (var i=0; i<graphs.length; i++) {
            myChartNofGnssSv.data.datasets[i].data = graphs[i].values;
            myChartNofGnssSv.data.datasets[i].showLine = true;
            myChartNofGnssSv.data.datasets[i].label = graphs[i].title;
        }
        myChartNofGnssSv.update();

    });
}


function init_data(props) {
    
    var data = {
            labels: [],
            datasets : []
        };

    props.forEach(function(element)
    {       
        var common_props = {
                    steppedLine: 'before', 
                    label: null,
                    fill: false,
                    lineTension: 0,
                    backgroundColor: element.color,
                    borderColor: element.color,
                    borderDash: [],
                    borderDashOffset: 0.0,
                    borderCapStyle : "round",
                    borderWidth : 1,
                    pointBorderColor: element.color,
                    pointBackgroundColor: "#00000000",
                    pointBorderWidth: 1,
                    pointHoverRadius: 1,
                    pointHoverBackgroundColor: "#00000000",
                    pointHoverBorderColor: element.color,
                    pointHoverBorderWidth: 5,
                    pointRadius: 0,
                    data: []
                }
        var props = $.extend( {}, common_props ) // makes a copy of current common props
        data.datasets.push(props)
    });
    
    return data;
}


$(document).ready(function() {
    
    var chartNofGnssSv = document.getElementById('ChartNofGnssSv');
    
    var options = {
    		responsive: true,
            animation: false,
            scales: {
                xAxes: [{
                    scaleLabel: { display: true, labelString: 'UTC' }
                }],
                yAxes: [{
                    ticks: { min: 0 }
                }]
            },
            legend: {
                labels: {
                    filter: function(item, chart) {
                        // Logic to remove a particular legend item goes here
                        return item.text!=null;
                    }
                }
            }
        };
    
    var data = init_data ( [{ color: "#000000ff" },
    	                    { color: "#ff0000ff" },
    	                    { color: "#4aa835ff" }
                           ]);

    // Chart options
    Chart.defaults.global.tooltips.enabled = false;

    myChartNofGnssSv = Chart.Line(chartNofGnssSv,{
        data:data,
        options:options
    });

    function loop() {
        updateTable();
        var delay = Math.max(1000, refresh_time*1000);
        setTimeout( loop, delay );
      }

    loop();

});

